print("Hot Potato Killer Loading.")

--===Variables===
local runIntThkrOnce = true -- used to run code in the IntermissionThinker hook once

--variables that users can change. On = 1, Off = 0
local IntermissionTeamChange = CV_RegisterVar({ --make a console variable to control whether intermission teamchange is blocked or not
	name = "tu_intermissionteamchange",
	defaultvalue = "Off",
	flags = CV_NETVAR,
	PossibleValue = CV_OnOff
})

local forceJoin = CV_RegisterVar({ --make a console variable to control whether players are forced to join the game during intermission.
	name = "tu_forceintermissionjoin",
	defaultvalue = "Off",
	flags = CV_NETVAR,
	PossibleValue = CV_OnOff
})


--===Functions===

local function NoSpecMessage(action, player) --makes the response string and sends it to the user. action is the string, player is the user object
	local strMes = "Sorry, you can't " .. action .. " right now."
	chatprintf(player, strMes, true)
end

--main function
local function StopAllowingTagBullshit(player) --player is the user object
	
	--Check if we're in an intermission (this code block prevents a warning)
	if (TagUtilities.inIntermission == true) and (forceJoin.value == 1 or IntermissionTeamChange.value == 0) then  --If intermissionTeamChange is off and we're in an intermission, it will block a team change. forceJoin has priority over intermissionteamChange.
		NoSpecMessage("change teams", player) --send the user a response
		return false --Stop user from changing in intermission
	elseif TagUtilities.inIntermission == true then --Only check if we're in an intermission
		return true --Allow user to change in intermission
	end
	
	--Check if this is indeed tag that we're playing
	if not TagUtilities.isTag then
		return nil --Not tag, return nil and stop the whole hook
	end
	--If the code continues, then this is indeed tag.
	
	local taggers = 0 --Integer to hold the number of current taggers
	TagUtilities.HideTimeDiff = leveltime - (CV_FindVar('hidetime').value * TICRATE) --Value to store the difference between the current elapsed time and hidetime in tics.
	--If the value of currentHideTime is positive, then hidetime is over.
	
	--This for loop goes through every player and checks if they are in the tag team
	for person in players.iterate do
		if (person.pflags & PF_TAGIT) then
			taggers = taggers + 1 --each tagger adds to the count, allows us to make a comparison later
		end
	end
	
	if ((TagUtilities.HideTimeDiff < 0) or (taggers <= 1)) then --Checking if the hide time is still on or if there's less than or equal to one tagger. Taggers shouldn't be able to spectate if either is true.
		if (player.pflags & PF_TAGIT) then --Checking if the individual player is a tagger. Hiders should be allowed to spectate whenever, this helps to exclusively block taggers.
			NoSpecMessage("spectate", player) --Sends the noSpecMessage string to only the user that tried to change teams, boolean enables a sound to play.
			return false --Do not allow the team change
		end
	end
	return true
end

--force players to join if forceJoin is enabled
local function forceThoseFuckersToPlay() --nuclear option for when people are insistant on using spectate cheese, stops spectating before a match ends to avoid being it.
	if forceJoin.value == 1 then
		for person in players.iterate do
			person.spectator = false
		end
	end
end


--===Game hooks===

--this hook is basically being used to sync variables
addHook("NetVars", function(network)
	TagUtilities.inIntermission = network(TagUtilities.inIntermission)
end)

--Hook handles all code every time a team switch is attempted
addHook("TeamSwitch", StopAllowingTagBullshit) --run the main function each time a teamswitch is attempted

--This code block decides if the game is in an intermission or not.
addHook("IntermissionThinker", do --this hook runs every tick
		TagUtilities.inIntermission = true --If an intermission is triggered, set the value to true
		if runIntThkrOnce == true then --run this function once during the intermission
			forceThoseFuckersToPlay() --Start the force play function, if forceJoin is disabled it will return nil
			runIntThkrOnce = false
		end
end)

addHook("MapLoad", do
	TagUtilities.inIntermission = false --If a map is loaded, set the value to false
	runIntThkrOnce = true --Set value to true now that we left the intermission
end)

print("Hot Potato Killer is finished Loading.")